package com.okx.open.api.test.sub.account;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson2.JSONObject;
import com.okx.open.api.bean.sub.account.param.CreateSubAccountApikey;
import com.okx.open.api.bean.sub.account.param.DelSunAccountApikey;
import com.okx.open.api.bean.sub.account.param.ReSetSubAccountApikey;
import com.okx.open.api.bean.sub.account.param.SubAccountTransfer;
import com.okx.open.api.service.sub.account.SubAccountApiService;
import com.okx.open.api.service.sub.account.impl.SubAccountApiServiceImpl;

/**
 * 
 * @author spb512
 * @date 2022年6月5日 下午5:02:59
 *
 */
public class SubAccountApiTests extends SubAccountApiBaseTest {

	Logger logger = LoggerFactory.getLogger(getClass());

	private SubAccountApiService subAccountApiService;

	@BeforeEach
	public void before() {
		this.config = this.config();
		this.subAccountApiService = new SubAccountApiServiceImpl(this.config);
	}

	/**
	 * 母账户查询子账户的交易账户详细资产余额（适用于母账户） Query detailed balance info of Trading Account of
	 * a sub-account via the master account (applies to master accounts only) GET
	 * /api/v5/account/subaccount/balances
	 */
	@Test
	public void testGetSubAccountBalances() {
		JSONObject result = this.subAccountApiService.getSubAccountBalances("ctt042501");
		toResultString(logger, "result", result);
	}

	/**
	 * 子账户转账记录（仅适用于母账户） View sub-account list(applies to master accounts only) GET
	 * /api/v5/asset/subaccount/bills
	 */
	@Test
	public void testGetSubAccountBills() {
		JSONObject result = this.subAccountApiService.getSubAccountBills("OKB", "0", "ctt042501", null, null, null);
		toResultString(logger, "result", result);
	}

	/**
	 * 删除子账户的Apikey（仅适用于母账户） Delete the Apikey of sub-accounts (applies to master
	 * accounts only) POST /api/v5/users/subaccount/delete-apikey
	 */
	@Test
	public void testDelSubAccountApikey() {
		DelSunAccountApikey delSunAccountApikey = new DelSunAccountApikey();

		delSunAccountApikey.setPwd("");
		delSunAccountApikey.setSubAcct("");
		delSunAccountApikey.setApiKey("");
		JSONObject result = this.subAccountApiService.delSubAccountApiKey(delSunAccountApikey);
		toResultString(logger, "result", result);

	}

	/**
	 * 重置子账户的Apikey（仅适用于母账户） Reset the Apikey of a sub-account(applies to master
	 * accounts only) POST /api/v5/users/subaccount/modify-apikey
	 */
	@Test
	public void testReSetSubAccountApikey() {
		ReSetSubAccountApikey reSetSubAccountApikey = new ReSetSubAccountApikey();

		reSetSubAccountApikey.setPwd("");
		reSetSubAccountApikey.setSubAcct("");
		reSetSubAccountApikey.setLabel("11111");
		reSetSubAccountApikey.setApiKey("");
		reSetSubAccountApikey.setPerm("read_only");
		reSetSubAccountApikey.setIp(null);

		JSONObject result = this.subAccountApiService.reSetSubAccountApiKey(reSetSubAccountApikey);
		toResultString(logger, "result", result);

	}

	/**
	 * 创建子账户的Apikey（仅适用于母账户）Create an Apikey for a sub-account(applies to master
	 * accounts only) POST /api/v5/users/subaccount/apikey
	 */
	@Test
	public void testCreateSubAccountApikey() {
		CreateSubAccountApikey createSubAccountApikey = new CreateSubAccountApikey();

		createSubAccountApikey.setPwd("");
		createSubAccountApikey.setSubAcct("");
		createSubAccountApikey.setLabel("create123");
		createSubAccountApikey.setPassphrase("");
		createSubAccountApikey.setPerm("read_only");
		createSubAccountApikey.setIp(null);

		JSONObject result = this.subAccountApiService.createSubAccountApiKey(createSubAccountApikey);
		toResultString(logger, "result", result);
	}

	/**
	 * 查看子账户列表（仅适用于母账户） View sub-account list(applies to master accounts only) GET
	 * /api/v5/users/subaccount/list
	 */
	@Test
	public void testGetSubAccountList() {
		JSONObject result = this.subAccountApiService.getSubAccountList("", "", "", "", "");
		toResultString(logger, "result", result);
	}

	/**
	 * 母账户控制子账户与子账户之间划转（仅适用于母账户） Master accounts manage the transfers between
	 * sub-accounts(applies to master accounts only) POST
	 * /api/v5/asset/subaccount/transfer
	 */
	@Test
	public void testSubAccountTransfer() {
		SubAccountTransfer subAccountTransfer = new SubAccountTransfer();
		subAccountTransfer.setCcy("OKB");
		subAccountTransfer.setAmt("1");
		subAccountTransfer.setForm("6");
		subAccountTransfer.setTo("6");
		subAccountTransfer.setFromSubAccount("");
		subAccountTransfer.setToSubAccount("");

		JSONObject result = this.subAccountApiService.subAccountTransfer(subAccountTransfer);
		toResultString(logger, "result", result);
	}

}
